@@ -30,6 +30,7 @@
 
 #include <gdbus.h>
 
+#include "dbus.h"
 #include "log.h"
 
 #include "transfer.h"
@@ -41,7 +42,8 @@
 	"\xF9\xEC\x7B\xC4\x95\x3C\x11\xD2\x98\x4E\x52\x54\x00\xDC\x9E\x09"
 #define OBEX_FTP_UUID_LEN 16
 
-#define FTP_INTERFACE "org.openobex.FileTransfer"
+#define FTP_INTERFACE "org.bluez.obex.FileTransfer"
+#define ERROR_INTERFACE "org.bluez.obex.Error"
 #define FTP_UUID "00001106-0000-1000-8000-00805f9b34fb"
 #define PCSUITE_UUID "00005005-0000-1000-8000-0002ee000001"
 
@@ -57,7 +59,7 @@ static void async_cb(struct obc_session *session, struct obc_transfer *transfer,
 	DBusMessage *reply, *msg = user_data;
 
 	if (err != NULL)
-		reply = g_dbus_create_error(msg, "org.openobex.Error.Failed",
+		reply = g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 	else
 		reply = dbus_message_new_method_return(msg);
@@ -78,13 +80,13 @@ static DBusMessage *change_folder(DBusConnection *connection,
 				DBUS_TYPE_STRING, &folder,
 				DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments", NULL);
+				ERROR_INTERFACE ".InvalidArguments", NULL);
 
 	obc_session_setpath(session, folder, async_cb, message, &err);
 	if (err != NULL) {
 		DBusMessage *reply;
 		reply =  g_dbus_create_error(message,
-						"org.openobex.Error.Failed",
+						ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 		g_error_free(err);
 		return reply;
@@ -95,39 +97,6 @@ static DBusMessage *change_folder(DBusConnection *connection,
 	return NULL;
 }
 
-static void append_variant(DBusMessageIter *iter, int type, void *val)
-{
-	DBusMessageIter value;
-	char sig[2] = { type, '\0' };
-
-	dbus_message_iter_open_container(iter, DBUS_TYPE_VARIANT, sig, &value);
-
-	dbus_message_iter_append_basic(&value, type, val);
-
-	dbus_message_iter_close_container(iter, &value);
-}
-
-static void dict_append_entry(DBusMessageIter *dict,
-			const char *key, int type, void *val)
-{
-	DBusMessageIter entry;
-
-	if (type == DBUS_TYPE_STRING) {
-		const char *str = *((const char **) val);
-		if (str == NULL)
-			return;
-	}
-
-	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
-							NULL, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	append_variant(&entry, type, val);
-
-	dbus_message_iter_close_container(dict, &entry);
-}
-
 static void xml_element(GMarkupParseContext *ctxt,
 			const gchar *element,
 			const gchar **names,
@@ -147,7 +116,7 @@ static void xml_element(GMarkupParseContext *ctxt,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
-	dict_append_entry(&dict, "Type", DBUS_TYPE_STRING, &element);
+	obex_dbus_dict_append(&dict, "Type", DBUS_TYPE_STRING, &element);
 
 	/* FIXME: User, Group, Other permission must be reviewed */
 
@@ -157,9 +126,11 @@ static void xml_element(GMarkupParseContext *ctxt,
 		if (g_str_equal("Size", key) == TRUE) {
 			guint64 size;
 			size = g_ascii_strtoll(values[i], NULL, 10);
-			dict_append_entry(&dict, key, DBUS_TYPE_UINT64, &size);
+			obex_dbus_dict_append(&dict, key, DBUS_TYPE_UINT64,
+								&size);
 		} else
-			dict_append_entry(&dict, key, DBUS_TYPE_STRING, &values[i]);
+			obex_dbus_dict_append(&dict, key, DBUS_TYPE_STRING,
+								&values[i]);
 	}
 
 	dbus_message_iter_close_container(iter, &dict);
@@ -173,24 +144,6 @@ static const GMarkupParser parser = {
 	NULL
 };
 
-static void get_file_callback(struct obc_session *session,
-						struct obc_transfer *transfer,
-						GError *err, void *user_data)
-{
-	DBusMessage *msg = user_data;
-	DBusMessage *reply;
-
-	if (err)
-		reply = g_dbus_create_error(msg,
-					"org.openobex.Error.Failed",
-					"%s", err->message);
-	else
-		reply = dbus_message_new_method_return(msg);
-
-	g_dbus_send_message(conn, reply);
-	dbus_message_unref(msg);
-}
-
 static void list_folder_callback(struct obc_session *session,
 						struct obc_transfer *transfer,
 						GError *err, void *user_data)
@@ -236,13 +189,13 @@ static DBusMessage *create_folder(DBusConnection *connection,
 				DBUS_TYPE_STRING, &folder,
 				DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments", NULL);
+				ERROR_INTERFACE ".InvalidArguments", NULL);
 
 	obc_session_mkdir(session, folder, async_cb, message, &err);
 	if (err != NULL) {
 		DBusMessage *reply;
 		reply = g_dbus_create_error(message,
-				"org.openobex.Error.Failed",
+				ERROR_INTERFACE ".Failed",
 				"%s", err->message);
 		g_error_free(err);
 		return reply;
@@ -273,7 +226,7 @@ static DBusMessage *list_folder(DBusConnection *connection,
 	}
 
 fail:
-	reply = g_dbus_create_error(message, "org.openobex.Error.Failed", "%s",
+	reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed", "%s",
 								err->message);
 	g_error_free(err);
 	return reply;
@@ -294,20 +247,19 @@ static DBusMessage *get_file(DBusConnection *connection,
 				DBUS_TYPE_STRING, &source_file,
 				DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments", NULL);
+				ERROR_INTERFACE ".InvalidArguments", NULL);
 
 	transfer = obc_transfer_get(NULL, source_file, target_file, &err);
 	if (transfer == NULL)
 		goto fail;
 
-	if (obc_session_queue(session, transfer, get_file_callback, message,
-								&err)) {
-		dbus_message_ref(message);
-		return NULL;
-	}
+	if (!obc_session_queue(session, transfer, NULL, NULL, &err))
+		goto fail;
+
+	return obc_transfer_create_dbus_reply(transfer, message);
 
 fail:
-	reply = g_dbus_create_error(message, "org.openobex.Error.Failed", "%s",
+	reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed", "%s",
 								err->message);
 	g_error_free(err);
 	return reply;
@@ -328,7 +280,7 @@ static DBusMessage *put_file(DBusConnection *connection,
 					DBUS_TYPE_STRING, &targetfile,
 					DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments",
+				ERROR_INTERFACE ".InvalidArguments",
 				"Invalid arguments in method call");
 
 	transfer = obc_transfer_put(NULL, targetfile, sourcefile, NULL, 0,
@@ -336,11 +288,13 @@ static DBusMessage *put_file(DBusConnection *connection,
 	if (transfer == NULL)
 		goto fail;
 
-	if (obc_session_queue(session, transfer, NULL, NULL, &err))
-		return dbus_message_new_method_return(message);
+	if (!obc_session_queue(session, transfer, NULL, NULL, &err))
+		goto fail;
+
+	return obc_transfer_create_dbus_reply(transfer, message);
 
 fail:
-	reply = g_dbus_create_error(message, "org.openobex.Error.Failed", "%s",
+	reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed", "%s",
 								err->message);
 	g_error_free(err);
 	return reply;
@@ -359,14 +313,13 @@ static DBusMessage *copy_file(DBusConnection *connection,
 				DBUS_TYPE_STRING, &destname,
 				DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments", NULL);
+				ERROR_INTERFACE ".InvalidArguments", NULL);
 
 	obc_session_copy(session, filename, destname, async_cb, message, &err);
 	if (err != NULL) {
 		DBusMessage *reply;
-		reply = g_dbus_create_error(message,
-						"org.openobex.Error.Failed",
-						"%s", err->message);
+		reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed",
+							"%s", err->message);
 		g_error_free(err);
 		return reply;
 	}
@@ -389,14 +342,13 @@ static DBusMessage *move_file(DBusConnection *connection,
 				DBUS_TYPE_STRING, &destname,
 				DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments", NULL);
+				ERROR_INTERFACE ".InvalidArguments", NULL);
 
 	obc_session_move(session, filename, destname, async_cb, message, &err);
 	if (err != NULL) {
 		DBusMessage *reply;
-		reply = g_dbus_create_error(message,
-						"org.openobex.Error.Failed",
-						"%s", err->message);
+		reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed",
+							"%s", err->message);
 		g_error_free(err);
 		return reply;
 	}
@@ -418,14 +370,13 @@ static DBusMessage *delete(DBusConnection *connection,
 				DBUS_TYPE_STRING, &file,
 				DBUS_TYPE_INVALID) == FALSE)
 		return g_dbus_create_error(message,
-				"org.openobex.Error.InvalidArguments", NULL);
+				ERROR_INTERFACE ".InvalidArguments", NULL);
 
 	obc_session_delete(session, file, async_cb, message, &err);
 	if (err != NULL) {
 		DBusMessage *reply;
-		reply = g_dbus_create_error(message,
-						"org.openobex.Error.Failed",
-						"%s", err->message);
+		reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed",
+							"%s", err->message);
 		g_error_free(err);
 		return reply;
 	}
@@ -435,23 +386,29 @@ static DBusMessage *delete(DBusConnection *connection,
 	return NULL;
 }
 
-static GDBusMethodTable ftp_methods[] = {
-	{ "ChangeFolder",	"s", "",	change_folder,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "CreateFolder",	"s", "",	create_folder,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "ListFolder",		"", "aa{sv}",	list_folder,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "GetFile",		"ss", "",	get_file,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "PutFile",		"ss", "",	put_file,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "CopyFile",		"ss", "",	copy_file,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "MoveFile",		"ss", "",	move_file,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "Delete",		"s", "",	delete,
-						G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable ftp_methods[] = {
+	{ GDBUS_ASYNC_METHOD("ChangeFolder",
+		GDBUS_ARGS({ "folder", "s" }), NULL, change_folder) },
+	{ GDBUS_ASYNC_METHOD("CreateFolder",
+		GDBUS_ARGS({ "folder", "s" }), NULL, create_folder) },
+	{ GDBUS_ASYNC_METHOD("ListFolder",
+		NULL, GDBUS_ARGS({ "folderinfo", "aa{sv}" }), list_folder) },
+	{ GDBUS_METHOD("GetFile",
+		GDBUS_ARGS({ "targetfile", "s" }, { "sourcefile", "s" }),
+		GDBUS_ARGS({ "transfer", "o" }, { "properties", "a{sv}" }),
+		get_file) },
+	{ GDBUS_METHOD("PutFile",
+		GDBUS_ARGS({ "sourcefile", "s" }, { "targetfile", "s" }),
+		GDBUS_ARGS({ "transfer", "o" }, { "properties", "a{sv}" }),
+		put_file) },
+	{ GDBUS_ASYNC_METHOD("CopyFile",
+		GDBUS_ARGS({ "sourcefile", "s" }, { "targetfile", "s" }), NULL,
+		copy_file) },
+	{ GDBUS_ASYNC_METHOD("MoveFile",
+		GDBUS_ARGS({ "sourcefile", "s" }, { "targetfile", "s" }), NULL,
+		move_file) },
+	{ GDBUS_ASYNC_METHOD("Delete",
+		GDBUS_ARGS({ "file", "s" }), NULL, delete) },
 	{ }
 };
 
